WintergraspFu = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "FuBarPlugin-2.0", "AceEvent-2.0")
WintergraspFu:RegisterDB("WintergraspFuDB")
WintergraspFu:RegisterDefaults('profile', {
		showestimate = true
	}
)

local L = AceLibrary("AceLocale-2.2"):new("WintergraspFu")

local BATTLE_DURATION = 30
local BETWEEN_BATTLE = 150

local options = {
	type = "group",
	args = {
		showestimate = {
			type = 'toggle',
			name = L["Show estimate"],
			desc = L["Show best estimate when exact time is not available."],
			get = function() return WintergraspFu.db.profile.showestimate end,
			set = function() WintergraspFu.db.profile.showestimate = not WintergraspFu.db.profile.showestimate end,
		}
	}
}

WintergraspFu.OnMenuRequest = options
WintergraspFu.defaultPosition = "LEFT"




--
--  Startup, enabling and disabling
--

function WintergraspFu:OnInitialize()
end

function WintergraspFu:OnEnable()
	self:RegisterEvent("UPDATE_WORLD_STATES", "CheckZone")
	self:RegisterEvent("CHAT_MSG_ADDON", "AddonMessage")
	self:RegisterEvent("CHAT_MSG_MONSTER_YELL", "MonsterYell")
	self:RegisterEvent("GOSSIP_SHOW", "GossipShow")
	self:ScheduleRepeatingEvent(self.EverySecond, 1, self)
	if self.db.realm.nextbattle == nil or self.db.realm.nextbattle < time() - 600 then
		if IsInGuild() then
			SendAddonMessage("WgFu", "?", "GUILD")
		end
	end
end

function WintergraspFu:OnDisable()

end






--
--  Synchronization with guild
--   

function WintergraspFu:AddonMessage(arg1, arg2, arg3, arg4)
-- Accept synchronization from BangWintergrasp
	if arg1 == "BangWG5" and arg4 ~= UnitName("player") then
		local bangwgtime = tonumber(arg2);
		if bangwgtime > 60 and bangwgtime < 9000 then
			SetNextBattle(time() + bangwgtime, 10);
		end
	end
--
	if arg1 ~= "WgFu" then
		return
	end
--[===[@alpha@
	ChatFrame1:AddMessage("[" .. arg3 .. "]" .. arg4 .. " " .. arg1 .. ": " .. arg2)
--@end-alpha@]===]
	local posdiff = string.match(arg2, "%+(%d*)$")
	if posdiff ~= nil then
		self.db.realm.nextbattle = time() + posdiff;
		self.db.realm.lastsync = self.db.realm.nextbattle
	end
	if arg2 == "?" and self.db.realm.nextbattle ~= nil and self.db.realm.nextbattle > time() then
		SendAddonMessage("WgFu", "+" .. self.db.realm.nextbattle - time(), "WHISPER", arg4)
	end
end

function WintergraspFu:SyncNextBattle(newnextbattle)
	if IsInGuild() then
		SendAddonMessage("WgFu", "+" .. newnextbattle - time(), "GUILD")
	end
end

function WintergraspFu:SetNextBattle(newnextbattle, accuracy)
	if self.db.realm.lastsync == nil or math.abs(newnextbattle - self.db.realm.lastsync) > accuracy then
--[===[@alpha@
	ChatFrame1:AddMessage(L["WgFu: New estimated start at "] .. date("%H:%M:%S", newnextbattle))
--@end-alpha@]===]
		self:SyncNextBattle(newnextbattle)
		self.db.realm.lastsync = newnextbattle
	end
	self.db.realm.nextbattle = newnextbattle
end







--
--  Obtaining time from the yells in Dalaran
--

function WintergraspFu:MonsterYell(arg1, arg2)
	if arg2 == L["Arcanist Braedin"] or arg2 == L["Magister Surdiel"] then
		local m = string.match(arg1, L["The battle for control of Wintergrasp will begin in (%d*) minute"])
		if m ~= nil then
			self:SetNextBattle(time() + 60*m, 5)
			return
		end
		local h = string.match(arg1, L["The battle for control of Wintergrasp will begin in (%d*) hour"])
		if h ~= nil then
			self:SetNextBattle(time() + 3600*h, 5)
			return
		end
	end
end







--
--  Obtaining time from talking to the wintergrasp portal mage
--

function WintergraspFu:GossipShow()
	local text = GetGossipText()
	local h, m, s = string.match(text, L["The next battle begins in (%d):(%d*):(%d*)%."]);
	if s ~= nil then 
		local newnextbattle = time() + h*3600 + m*60 + s;
		self:SetNextBattle(newnextbattle, 10);
	end
end






--
--  Obtaining time from the zone state messages
--

local worldstatetext = nil
local worldstatetime = nil
function WintergraspFu:CheckZone()
	if (GetMapInfo()) ~= "LakeWintergrasp" then
		return
	end
	local uiType, state, text = GetWorldStateUIInfo(8)
	worldstatetext = text
	worldstatetime = time()
end

local lasttickworldstate = nil
local lastusedworldstate = nil
function WintergraspFu:DelayedCheckWorldState()
	if worldstatetext ~= nil and lasttickworldstate == worldstatetext and lastusedworldstate ~= worldstatetext then
		-- World state constant between two ticks and not same as last used; update!
		local h, m, s = string.match(worldstatetext, L["Next Battle: (%d):(%d%d):(%d%d)$"])
		if s ~= nil then 
			local newnextbattle = worldstatetime + h*3600 + m*60 + s;
			self:SetNextBattle(newnextbattle, 3);
		end
		lastusedworldstate = worldstatetext
	end
	lasttickworldstate = worldstatetext
end

  




--
-- Periodic updates of text + delayed state check
--

function WintergraspFu:EverySecond()
	self:UpdateText()
	self:DelayedCheckWorldState()
end

function WintergraspFu:OnTextUpdate()
	if self:IsTextColored() then
		self:SetText("|cffffffff"..L["WG"]..":|r |cffffff00" .. self:CreateStatusText() .. "|r")
	else
		self:SetText(L["WG"]..": " .. self:CreateStatusText())
	end
end

function WintergraspFu:GetTimeText(delta)
	if delta < 0 then 
		return L["Active"]
	end
	local h = math.floor(delta / 3600)
	local m = math.floor(delta % 3600 / 60)
	local s = delta % 60
	local text = ""
	if h > 0 then
		text = text .. h .. L["h"].." "
	end
	if m > 0 then
		text = text .. m .. L["m"].." "
	end
	text = text .. s .. L["s"]
	return text
end

function WintergraspFu:GetTimeText2(delta)
	if delta < 0 then 
		return L["Active"]
	end
	local h = math.floor(delta / 60)
	local m = math.floor(delta % 60)
	local text = ""
	if h > 0 then
		text = text .. h .. L["h"].." "
	end
	text = text .. m .. L["m"].." "
	return text
end

function WintergraspFu:CreateStatusText()
	if self.db.realm.nextbattle == nil then
		return L["Unknown"]
	end
	local diff = self.db.realm.nextbattle - time()
	if diff >= 0 then
		return self:GetTimeText(diff)
	end
	if not self.db.profile.showestimate then
		return L["Unknown"]
	end
	diff = math.floor(-diff/60)
	-- Estimating more than a few battles away is meaningless
	if diff > (BETWEEN_BATTLE+BATTLE_DURATION)*4 then 
		return L["Unknown"]
	end
	local longestimate = BETWEEN_BATTLE - (diff + BETWEEN_BATTLE) % (BATTLE_DURATION + BETWEEN_BATTLE) -- estimate for full length battles
	local shortestimate = BETWEEN_BATTLE - (diff + BETWEEN_BATTLE) % (BATTLE_DURATION/2 + BETWEEN_BATTLE) -- estimate for short battles
	return "~" .. self:GetTimeText2(shortestimate) .. " .. " .. self:GetTimeText2(longestimate)
end



